import os
from wesci.timer import Timer
from wesci.hash import Hash
import tempfile
from matplotlib.pyplot import gcf
from PIL import Image


class Figures(object):
    FIGURE_FORMAT = 'png'
    SIZE_PIXELS = 512, 512

    @staticmethod
    def add_output_figure(thumbnail_dir, fig):
        with Timer() as timer:
            fighash = Figures.__process_figure(thumbnail_dir, fig)
        return {'hash': fighash, 'timing': timer.duration_ms()}

    @staticmethod
    def __process_figure(thumbnail_dir, fig):
        if fig is None:
            fig = gcf()
        try:
            tmp, temp_filename = tempfile.mkstemp(suffix='.png')
            os.close(tmp)  # os may prohibit saving to an already open file
            fig.savefig(temp_filename)
            fighash = Figures.__figure_hash(temp_filename)
            Figures.__store_figure_thumbnail(temp_filename, thumbnail_dir,
                                             fighash)
        finally:
            os.remove(temp_filename)
        return fighash

    @staticmethod
    def __figure_hash(filename):
        with open(filename, 'rb') as f:
            file_data = f.read()
        return Hash.hash(file_data)

    @staticmethod
    def __save_figure_thumbnail(origin, thumbnail_dir, filename):
        fig = Image.open(origin)
        fig.thumbnail(Figures.SIZE_PIXELS)
        fig.save('%s/%s.%s' % (thumbnail_dir, filename, Figures.FIGURE_FORMAT))

    @staticmethod
    def __store_figure_thumbnail(origin, thumbnail_dir, filename):
        if not os.path.exists(thumbnail_dir):
            os.makedirs(thumbnail_dir)
        Figures.__save_figure_thumbnail(origin, thumbnail_dir, filename)



"""
This module support both Python 2.7 and 3 by handling importing packages
either unsupported or moved between Python versions
"""

############################
# StringIO
############################
try:
    from StringIO import StringIO
except ImportError:
    from io import StringIO



class Config(object):

    API_KEY = 'api_key'
    SECRET = 'secret'
    KEYS = set([API_KEY, SECRET])

    def load(self, conf_file_path):
        self.conf = {}

        with open(conf_file_path) as f:
            content = f.readlines()

        for line in content:
            self.conf.update(Config.process_conf_line(line))

        if not self.conf:   # old format conf
            self.conf[Config.API_KEY] = content[0].strip()

    def api_key(self):
        return self.conf[Config.API_KEY]

    def has_secret(self):
        return Config.SECRET in self.conf

    def secret(self):
        return self.conf[Config.SECRET]

    @staticmethod
    def process_conf_line(line):
        key = line.split('=')[0].strip()
        if key in Config.KEYS:
            return {key: line.split('=', 1)[1].strip()}
        return {}



from wesci.timer import Timer
from wesci.hash import Hash
from os.path import splitext
from csv import reader


class Files(object):

    PREVIEW_TEXT_LINES = 10
    PREVIEW_TEXT_LINE_WIDTH = 80
    PREVIEW_CSV_ROWS = 10
    PREVIEW_CSV_COLUMNS = 10

    @staticmethod
    def generate_data_for_log(files_dict):
        data_for_log = {}
        for name, path in files_dict.items():
            data, read_duration = Files.__get_file_data_and_duration_of_read(
                path)
            file_hash, hash_duration = Files.__calc_hash_and_duration(data)
            size = len(data)
            preview, preview_duration = Files.__get_preview(path)
            data_for_log[name] = Files.__format_data_for_log(
                path,
                file_hash,
                size,
                preview,
                read_duration,
                hash_duration,
                preview_duration)
        return data_for_log

    @staticmethod
    def __calc_hash_and_duration(file_data):
        with Timer() as timer:
            res = Hash.hash(file_data)
        return res, timer.duration_ms()

    @staticmethod
    def __get_file_data_and_duration_of_read(file_path):
        with Timer() as timer:
            with open(file_path, 'rb') as f:
                file_data = f.read()
        return file_data, timer.duration_ms()

    @staticmethod
    def __get_txt_preview(path):
        preview = []
        with open(path, 'r') as f:
            for line in f:
                preview.append(line[:Files.PREVIEW_TEXT_LINE_WIDTH])
                if len(preview) == Files.PREVIEW_TEXT_LINES:
                    break
        return '\n'.join(preview)

    @staticmethod
    def __get_csv_preview(path):
        preview = []
        with open(path, 'r') as f:
            rows = reader(f)
            for row in rows:
                preview.append(','.join(row[:Files.PREVIEW_CSV_COLUMNS]))
                if len(preview) == Files.PREVIEW_CSV_ROWS:
                    break
        return '\n'.join(preview)

    @staticmethod
    def __get_preview(path):
        with Timer() as timer:
            preview = ''
            t, ext = splitext(path)
            ext = ext.lower()
            if ext == '.txt':
                preview = Files.__get_txt_preview(path)
            if ext == '.json':
                preview = Files.__get_txt_preview(path)
            elif ext == '.csv':
                preview = Files.__get_csv_preview(path)
        return preview, timer.duration_ms()

    @staticmethod
    def __format_data_for_log(
            path,
            file_hash,
            size,
            preview,
            read_duration,
            hash_duration,
            preview_duration):
        return {'path': path,
                'hash': file_hash,
                'size': size,
                'preview': preview,
                'timing': {
                    'file_read_ms': read_duration,
                    'hash_calc_ms': hash_duration,
                    'preview_ms': preview_duration},
                }



"""
We-Sci's tools for researchers.

modules available:
==================

* Logger - Log input, code and outputs of python scientific research scripts,
           to be used with We-Sci's web platform (currently in closed beta)

Please email contact@we-sci.com for more info

"""
from wesci.logger import Logger
